part of 'vehicles_on_the_property.dart';

/// [VehicleHistoryScreen] is a StatefulWidget that displays a list of vehicles.

// ignore: must_be_immutable
class VehicleHistoryScreen extends StatefulWidget {
  VehicleHistoryScreen(this.model, {super.key});
  late VehicleModel model;

  @override
  State<VehicleHistoryScreen> createState() => _VehicleHistoryScreenState();
}

class _VehicleHistoryScreenState extends State<VehicleHistoryScreen> {
  late Future<void> _future = _fetchVisits();
  late ScrollController _scrollController;
  List<dynamic> _wrap = [];
  int _pageNumber = 1;
  bool _hasData = false;
  bool _loading = false;

  int count = 0;

  @override
  void initState() {
    super.initState();

    // widget.model = createModel(context, () => VehicleModel());
    _future = _fetchVisits();
    _scrollController = ScrollController()
      ..addListener(() {
        if (_scrollController.position.atEdge &&
            _scrollController.position.pixels != 0) {
          _loadMore();
        }
      });
  }

  void _showNoMoreDataSnackBar(BuildContext context) {
    ScaffoldMessenger.of(context).showSnackBar(
      SnackBar(
        content: Text(
          FFLocalizations.of(context).getVariableText(
            ptText: "Não há mais dados.",
            enText: "No more data.",
          ),
          style: TextStyle(
            color: Colors.white,
            fontSize: LimitedFontSizeUtil.getBodyFontSize(context),
          ),
        ),
        duration: const Duration(seconds: 3),
        backgroundColor: FlutterFlowTheme.of(context).primary,
      ),
    );
  }

  String mockRandomMapStatusColor() {
    var statuses = [
      'ATI',
      'INA',
      'APR_CREATE',
      'APR_DELETE',
      'APR_UPDATE',
      'AGU_CHANGE'
    ];
    statuses.shuffle();
    return statuses.first;
  }

  Future<Widget> _item(BuildContext context, Map<String, dynamic> uItem) async {
    // uItem['status'] = mockRandomMapStatusColor();
    final bool? isOwner = uItem['isOwnerVehicle'];
    late final IconData? iconData;
    late final FreCardIcon? cardIcon;

    if (isOwner is bool) {
      iconData = isOwner ? Symbols.no_crash : Symbols.directions_car;
      cardIcon = FreCardIcon(
          height: 50,
          width: 100,
          icon: Icon(iconData, size: 80, opticalSize: 10));
    }
    final String? tag = uItem['tag'];
    final bool containTag = tag.isNotNullAndEmpty;
    final Map<String, String> labelsHashMap = {
      '${FFLocalizations.of(context).getVariableText(ptText: "Placa", enText: "License Plate")}:':
          uItem['licensePlate'] ?? '',
      '${FFLocalizations.of(context).getVariableText(ptText: "Modelo", enText: "Model")}:':
          uItem['model'] ?? '',
      '${FFLocalizations.of(context).getVariableText(ptText: "Proprietário", enText: "Owner")}:':
          uItem['personName'] ?? '',
      if (containTag)
        '${FFLocalizations.of(context).getVariableText(ptText: "Tag", enText: "Tag")}:':
            uItem['tag'] ?? '',
    };
    final Map<String, Color>? statusHashMap =
        await widget.model.generateStatusColorMap(uItem, 2);
    final List<Map<String, Color>?> statusHashMapList = [
      await widget.model.generateStatusColorMap(uItem, 2)
    ];
    Future<void> onTapCardItemAction() async {
      final widgetFuture = widget.model.buildVehicleDetails(
        icon: isOwner is bool ? cardIcon : null,
        item: uItem,
        context: context,
        model: widget.model,
      );

      final dialogContent = await widgetFuture;

      await showDialog(
        useSafeArea: true,
        context: context,
        builder: (context) {
          return Dialog(
            alignment: Alignment.center,
            child: dialogContent,
          );
        },
      ).whenComplete(() {
        safeSetState(() {
          _pageNumber = 1;
          _wrap = [];
          _future = _fetchVisits();
        });
      }).catchError((e, s) {
        DialogUtil.errorDefault(context);
        LogUtil.requestAPIFailed(
            "proccessRequest.php", "", "Consulta de Veículos", e, s);
        safeSetState(() {
          _hasData = false;
          _loading = false;
        });
      });
    }

    final statusLinkedHashMap = statusHashMapList
        .map((map) => LinkedHashMap<String, Color>.from(map ?? {}))
        .toList();
    final length = statusLinkedHashMap.expand((e) => [e.length]);
    print('CardItemTemplateComponentWidget: ${length}');
    return CardItemTemplateComponentWidget(
      icon: cardIcon,
      labelsHashMap: labelsHashMap,
      statusHashMap: statusHashMapList,
      onTapCardItemAction: onTapCardItemAction,
      itemWidthFactor: length == 1 ? 0.25 : 0.50,
    );
    // double itemWidthFactor = statusHashMap.length == 1 ? 0.5 : 0.5;
    //       double itemWidthFactor;
    // if (statusCount >= 3 && statusCount % 3 != 0) {
    //   itemWidthFactor = (index % 3 == 2) ? 0.5 : 0.25;
    // } else if (statusCount == 3) {
    //   itemWidthFactor = (index == 2) ? 0.52 : 0.25;
    // } else {
    //   itemWidthFactor = statusCount == 1 ? 0.5 : 0.25;
    // }
  }

  Future<ApiCallResponse?> _fetchVisits() async {
    try {
      setState(() => _loading = true);

      var response =
          await PhpGroup.getVehiclesByProperty.call(_pageNumber.toString());

      final List<dynamic> vehicles = response.jsonBody['vehicles'] ?? [];
      safeSetState(() => count = response.jsonBody['total_rows'] ?? 0);

      if (vehicles.isNotEmpty) {
        setState(() {
          _wrap.addAll(vehicles);
          _hasData = true;
          _loading = false;
        });

        return response;
      }

      _showNoMoreDataSnackBar(context);

      setState(() {
        _hasData = false;
        _loading = false;
      });

      return null;
    } catch (e, s) {
      DialogUtil.errorDefault(context);
      LogUtil.requestAPIFailed(
          "proccessRequest.php", "", "Consulta de Veículo", e, s);
      setState(() {
        _hasData = false;
        _loading = false;
      });
    }
    return null;
  }

  void _loadMore() {
    if (_hasData == true) {
      _pageNumber++;

      _future = _fetchVisits();
    }
  }

  @override
  Widget build(BuildContext context) {
    late final limitedHeaderTextSize =
        LimitedFontSizeUtil.getHeaderFontSize(context);
    late final double limitedBodyTextSize =
        LimitedFontSizeUtil.getBodyFontSize(context);

    return Column(
      mainAxisSize: MainAxisSize.max,
      mainAxisAlignment: MainAxisAlignment.start,
      children: [
        if (_hasData == false && _pageNumber <= 1 && _loading == false)
          Expanded(
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              mainAxisSize: MainAxisSize.max,
              children: [
                Center(
                  child: Text(
                    FFLocalizations.of(context).getVariableText(
                      ptText: "Nenhum veículo encontrado!",
                      enText: "No vehicle found",
                    ),
                    style: TextStyle(
                      fontFamily: 'Nunito',
                      fontSize: limitedHeaderTextSize,
                    ),
                  ),
                )
              ],
            ),
          )
        else if (_hasData == true || _pageNumber >= 1)
          Expanded(
              child: FutureBuilder<void>(
            future: _future,
            builder: (context, snapshot) {
              return ListView.builder(
                  shrinkWrap: true,
                  physics: const BouncingScrollPhysics(),
                  controller: _scrollController,
                  itemCount: _wrap.length + 1,
                  itemBuilder: (context, index) {
                    if (index == 0) {
                      // Add your item here
                      return Padding(
                        padding: const EdgeInsets.only(right: 30, top: 10),
                        child: Text(
                          (widget.model.amountRegister == '0' ||
                                  widget.model.amountRegister == null)
                              ? ''
                              : "${FFLocalizations.of(context).getVariableText(ptText: "Quantidade de Pets: ", enText: "Amount of Vehicles: ")}${widget.model.amountRegister}/$count",
                          textAlign: TextAlign.right,
                          style: TextStyle(
                            fontFamily: 'Nunito',
                            fontSize: limitedBodyTextSize,
                          ),
                        ),
                      );
                    } else {
                      Map<String, dynamic> item = _wrap[index - 1];
                      return FutureBuilder<Widget>(
                        future: _item(context, item),
                        builder: (context, snapshot) {
                          if (snapshot.connectionState ==
                              ConnectionState.done) {
                            return snapshot.data ?? Container();
                          } else {
                            return Center(child: CircularProgressIndicator());
                          }
                        },
                      );
                    }
                  });
            },
          )),
        if (_hasData == true && _loading == true)
          Container(
            padding: const EdgeInsets.only(top: 15, bottom: 15),
            child: Center(
              child: CircularProgressIndicator(
                valueColor: AlwaysStoppedAnimation<Color>(
                  FlutterFlowTheme.of(context).primary,
                ),
              ),
            ),
          )
      ].addToStart(const SizedBox(height: 0)),
    );
  }
}
